@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2021. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -429,6 +429,7 @@
 static const byte digits_per_sint_lookup[36-1];
 static const byte digits_per_small_lookup[36-1];
 static const Sint largest_power_of_base_lookup[36-1];
+static const double lg2_lookup[36-1];
 
 static ERTS_INLINE byte get_digits_per_signed_int(Uint base) {
     return digits_per_sint_lookup[base-2];
@@ -442,6 +443,10 @@ static ERTS_INLINE Sint get_largest_power_of_base(Uint base) {
     return largest_power_of_base_lookup[base-2];
 }
 
+static ERTS_INLINE double lookup_log2(Uint base) {
+    return lg2_lookup[base - 2];
+}
+
 /*
 ** compare two number vectors
 */
@@ -668,27 +673,25 @@ static dsize_t I_mul(ErtsDigit* x, dsize_t xl, ErtsDigit* y, dsize_t yl, ErtsDig
 
 static dsize_t I_sqr(ErtsDigit* x, dsize_t xl, ErtsDigit* r)
 {
-    ErtsDigit d_next = *x;
     ErtsDigit d;
     ErtsDigit* r0 = r;
     ErtsDigit* s = r;
 
     if ((r + xl) == x)	/* "Inline" operation */
 	*x = 0;
-    x++;
 	
     while(xl--) {
-	ErtsDigit* y = x;
+	ErtsDigit* y;
 	ErtsDigit y_0 = 0, y_1 = 0, y_2 = 0, y_3 = 0;
 	ErtsDigit b0, b1;
 	ErtsDigit z0, z1, z2;
 	ErtsDigit t;
 	dsize_t y_l = xl;
-		
+
+        d = *x;
+        x++;
+        y = x;
 	s = r;
-	d = d_next;
-	d_next = *x; 
-	x++;
 
 	DMUL(d, d, b1, b0);
 	DSUMc(*s, b0, y_3, t);
@@ -1159,8 +1162,11 @@ static dsize_t I_band(ErtsDigit* x, dsize_t xl, short xsgn,
 		*r++ = ~c1 & ~c2;
 		x++; y++;
 	    }
-	    while(xl--)
-		*r++ = ~*x++;
+	    while(xl--) {
+                DSUBb(*x,0,b1,c1);
+                *r++ = ~c1;
+		x++;
+            }
 	}
     }
     return I_btrail(r0, r, sign);
@@ -1642,21 +1648,16 @@ big_to_double(Wterm x, double* resp)
     ErtsDigit* s = BIG_V(xp) + xl;
     short xsgn = BIG_SIGN(xp);
     double dbase = ((double)(D_MASK)+1);
-#ifndef NO_FPE_SIGNALS 
-    volatile unsigned long *fpexnp = erts_get_current_fp_exception();
-#endif
-    __ERTS_SAVE_FP_EXCEPTION(fpexnp);
 
-    __ERTS_FP_CHECK_INIT(fpexnp);
     while (xl--) {
-	d = d * dbase + *--s;
+        d = d * dbase + *--s;
 
-	__ERTS_FP_ERROR(fpexnp, d, __ERTS_RESTORE_FP_EXCEPTION(fpexnp); return -1);
+        if (!erts_isfinite(d)) {
+            return -1;
+        }
     }
 
     *resp = xsgn ? -d : d;
-    __ERTS_FP_ERROR(fpexnp,*resp,;);
-    __ERTS_RESTORE_FP_EXCEPTION(fpexnp);
     return 0;
 }
 
@@ -1705,9 +1706,6 @@ double_to_big(double x, Eterm *heap, Uint hsz)
 	xp[i] = d; /* store digit */
 	x -= d; /* remove integer part */
     }
-    while ((ds & (BIG_DIGITS_PER_WORD - 1)) != 0) {
-	xp[ds++] = 0;
-    }
 
     if (is_negative) {
 	*hp = make_neg_bignum_header(sz-1);
@@ -1719,23 +1717,23 @@ double_to_big(double x, Eterm *heap, Uint hsz)
 
 
 /*
- ** Estimate the number of decimal digits (include sign)
+ ** Estimate the number of digits in given base (include sign)
  */
-int big_decimal_estimate(Wterm x)
+int big_integer_estimate(Wterm x, Uint base)
 {
     Eterm* xp = big_val(x);
     int lg = I_lg(BIG_V(xp), BIG_SIZE(xp));
-    int lg10 = ((lg+1)*28/93)+1;
+    int lgBase = ((lg + 1) / lookup_log2(base)) + 1;
 
-    if (BIG_SIGN(xp)) lg10++;	/* add sign */
-    return lg10+1;		/* add null */
+    if (BIG_SIGN(xp)) lgBase++;	/* add sign */
+    return lgBase + 1;		/* add null */
 }
 
 /*
-** Convert a bignum into a string of decimal numbers
+** Convert a bignum into a string of numbers in given base
 */
-
-static Uint write_big(Wterm x, void (*write_func)(void *, char), void *arg)
+static Uint write_big(Wterm x, int base, void (*write_func)(void *, char),
+                      void *arg)
 {
     Eterm* xp = big_val(x);
     ErtsDigit* dx = BIG_V(xp);
@@ -1743,48 +1741,72 @@ static Uint write_big(Wterm x, void (*write_func)(void *, char), void *arg)
     short sign = BIG_SIGN(xp);
     ErtsDigit rem;
     Uint n = 0;
-    const Uint digits_per_Sint = get_digits_per_signed_int(10);
-    const Sint largest_pow_of_base = get_largest_power_of_base(10);
+    const Uint digits_per_Sint = get_digits_per_signed_int(base);
+    const Sint largest_pow_of_base = get_largest_power_of_base(base);
 
     if (xl == 1 && *dx < largest_pow_of_base) {
-	rem = *dx;
-	if (rem == 0) {
-	    (*write_func)(arg, '0'); n++;
-	} else {
-	    while(rem) {
-		(*write_func)(arg, (rem % 10) + '0'); n++;
-		rem /= 10;
-	    }
-	}
+        rem = *dx;
+        if (rem == 0) {
+            (*write_func)(arg, '0'); n++;
+        } else {
+            while(rem) {
+                int digit = rem % base;
+
+                if (digit < 10) {
+                    (*write_func)(arg, digit + '0'); n++;
+                } else {
+                    (*write_func)(arg, 'A' + (digit - 10)); n++;
+                }
+
+                rem /= base;
+            }
+        }
     } else {
-	ErtsDigit* tmp  = (ErtsDigit*) erts_alloc(ERTS_ALC_T_TMP,
-					      sizeof(ErtsDigit)*xl);
-	dsize_t tmpl = xl;
+        ErtsDigit* tmp = (ErtsDigit*) erts_alloc(ERTS_ALC_T_TMP,
+                                                 sizeof(ErtsDigit) * xl);
+        dsize_t tmpl = xl;
 
-	MOVE_DIGITS(tmp, dx, xl);
+        MOVE_DIGITS(tmp, dx, xl);
 
-	while(1) {
+        while(1) {
             tmpl = D_div(tmp, tmpl, largest_pow_of_base, tmp, &rem);
-	    if (tmpl == 1 && *tmp == 0) {
-		while(rem) {
-		    (*write_func)(arg, (rem % 10)+'0'); n++;
-		    rem /= 10;
-		}
-		break;
-	    } else {
+
+            if (tmpl == 1 && *tmp == 0) {
+                while(rem) {
+                    int digit = rem % base;
+
+                    if (digit < 10) {
+                        (*write_func)(arg, digit + '0'); n++;
+                    } else {
+                        (*write_func)(arg, 'A' + (digit - 10)); n++;
+                    }
+
+                    rem /= base;
+                }
+                break;
+            } else {
                 Uint i = digits_per_Sint;
-		while(i--) {
-		    (*write_func)(arg, (rem % 10)+'0'); n++;
-		    rem /= 10;
-		}
-	    }
-	}
-	erts_free(ERTS_ALC_T_TMP, (void *) tmp);
+
+                while(i--) {
+                    int digit = rem % base;
+
+                    if (digit < 10) {
+                        (*write_func)(arg, digit + '0'); n++;
+                    } else {
+                        (*write_func)(arg, 'A' + (digit - 10)); n++;
+                    }
+
+                    rem /= base;
+                }
+            }
+        }
+        erts_free(ERTS_ALC_T_TMP, (void *) tmp);
     }
 
     if (sign) {
-	(*write_func)(arg, '-'); n++;
+        (*write_func)(arg, '-'); n++;
     }
+
     return n;
 }
 
@@ -1801,12 +1823,12 @@ write_list(void *arg, char c)
     blp->hp += 2;
 }
 
-Eterm erts_big_to_list(Eterm x, Eterm **hpp)
+Eterm erts_big_to_list(Eterm x, int base, Eterm **hpp)
 {
     struct big_list__ bl;
     bl.hp = *hpp;
     bl.res = NIL;
-    write_big(x, write_list, (void *) &bl);
+    write_big(x, base, write_list, (void *) &bl);
     *hpp = bl.hp;
     return bl.res;
 }
@@ -1817,11 +1839,11 @@ write_string(void *arg, char c)
     *(--(*((char **) arg))) = c;
 }
 
-char *erts_big_to_string(Wterm x, char *buf, Uint buf_sz)
+char *erts_big_to_string(Wterm x, int base, char *buf, Uint buf_sz)
 {
     char *big_str = buf + buf_sz - 1;
     *big_str = '\0';
-    write_big(x, write_string, (void *) &big_str);
+    write_big(x, base, write_string, (void*)&big_str);
     ASSERT(buf <= big_str && big_str <= buf + buf_sz - 1);
     return big_str;
 }
@@ -1830,11 +1852,11 @@ char *erts_big_to_string(Wterm x, char *buf, Uint buf_sz)
  * e.g. 1 bsl 64 -> "18446744073709551616"
  */
 
-Uint erts_big_to_binary_bytes(Eterm x, char *buf, Uint buf_sz)
+Uint erts_big_to_binary_bytes(Eterm x, int base, char *buf, Uint buf_sz)
 {
     char *big_str = buf + buf_sz;
     Uint n;
-    n = write_big(x, write_string, (void *) &big_str);
+    n = write_big(x, base, write_string, (void *) &big_str);
     ASSERT(buf <= big_str && big_str <= buf + buf_sz);
     return n;
 }
@@ -1924,7 +1946,7 @@ dsize_t big_bytes(Eterm x)
 ** xsz is the number of bytes in xp
 ** *r is untouched if number fits in small
 */
-Eterm bytes_to_big(byte *xp, dsize_t xsz, int xsgn, Eterm *r)
+Eterm bytes_to_big(const byte *xp, dsize_t xsz, int xsgn, Eterm *r)
 {
     ErtsDigit* rwp = BIG_V(r);
     dsize_t rsz = 0;
@@ -2146,6 +2168,24 @@ term_to_Uint64(Eterm term, Uint64 *up)
 #endif
 }
 
+int
+term_to_Uint32(Eterm term, Uint32 *up)
+{
+#if ERTS_SIZEOF_ETERM == 4
+    return term_to_Uint(term,up);
+#else
+    if (is_small(term)) {
+	Sint i = signed_val(term);
+	if (i >= 0) {
+            *up = (Uint32) i;
+            return 1;
+        }
+    }
+    *up = BADARG;
+    return 0;
+#endif
+}
+
 
 int term_to_Sint(Eterm term, Sint *sp)
 {
@@ -2336,11 +2376,55 @@ Eterm big_times(Eterm x, Eterm y, Eterm *r)
     return big_norm(r, rsz, sign);
 }
 
+/*
+** Fused div_rem for bignums
+*/
+int big_div_rem(Eterm lhs, Eterm rhs,
+                Eterm *q_hp, Eterm *q,
+                Eterm *r_hp, Eterm *r)
+{
+    Eterm *lhs_val = big_val(lhs);
+    Eterm *rhs_val = big_val(rhs);
 
-/* 
+    int div_sign = BIG_SIGN(lhs_val) != BIG_SIGN(rhs_val);
+    int rem_sign = BIG_SIGN(lhs_val);
+
+    dsize_t lhs_size = BIG_SIZE(lhs_val);
+    dsize_t rhs_size = BIG_SIZE(rhs_val);
+
+    dsize_t quotient_size, remainder_size;
+    Eterm quotient, remainder;
+
+    if (rhs_size == 1) {
+        quotient_size = D_div(BIG_V(lhs_val), lhs_size, BIG_DIGIT(rhs_val, 0),
+                              BIG_V(q_hp), BIG_V(r_hp));
+        remainder_size = 1;
+    } else {
+        quotient_size = I_div(BIG_V(lhs_val), lhs_size,
+                              BIG_V(rhs_val), rhs_size,
+                              BIG_V(q_hp), BIG_V(r_hp),
+                              &remainder_size);
+    }
+
+    quotient = big_norm(q_hp, quotient_size, div_sign);
+    if (quotient == NIL) {
+        return 0;
+    }
+
+    remainder = big_norm(r_hp, remainder_size, rem_sign);
+    if (remainder == NIL) {
+        return 0;
+    }
+
+    *q = quotient;
+    *r = remainder;
+
+    return 1;
+}
+
+/*
 ** Divide bignums
 */
-
 Eterm big_div(Eterm x, Eterm y, Eterm *q)
 {
     Eterm* xp = big_val(x);
@@ -2577,9 +2661,6 @@ static const double lg2_lookup[36-1] = {
     4.32193, 4.39232, 4.45943, 4.52356, 4.58496, 4.64386, 4.70044, 4.75489,
     4.80735, 4.85798, 4.90689, 4.9542, 5.0, 5.04439, 5.08746, 5.12928, 5.16993
 };
-static ERTS_INLINE double lookup_log2(Uint base) {
-    return lg2_lookup[base - 2];
-}
 
 /*
  * How many digits can fit into a signed int (Sint) for given base, we take
