@@ -83,7 +83,9 @@
          ssh_hostkey_fingerprint_sha256/1,
          ssh_hostkey_fingerprint_sha384/1,
          ssh_hostkey_fingerprint_sha512/1,
-         ssh_hostkey_fingerprint_list/1
+         ssh_hostkey_fingerprint_list/1,
+
+         chk_known_hosts/1
         ]).
 
 -include_lib("common_test/include/ct.hrl").
@@ -103,7 +105,8 @@ all() ->
      {group, new_format},
      {group, option_space},
      {group, ssh_hostkey_fingerprint},
-     {group, ssh_public_key_decode_encode}
+     {group, ssh_public_key_decode_encode},
+     chk_known_hosts
     ].
 
 
@@ -116,14 +119,16 @@ all() ->
                     connect_dsa_to_ecdsa,
                     connect_ecdsa_to_rsa_sha2,
                     connect_ecdsa_to_dsa,
-                    connect_ecdsa_to_ecdsa
+                    connect_ecdsa_to_ecdsa,
+                    connect_dsa_to_ed25519,
+                    connect_ecdsa_to_ed25519,
+                    connect_rsa_sha2_to_ed25519,
+                    connect_dsa_to_ed448,
+                    connect_ecdsa_to_ed448,
+                    connect_rsa_sha2_to_ed448
                    ]).
 
--define(tests_new, [connect_dsa_to_ed25519,
-                    connect_dsa_to_ed448,
-                    connect_ecdsa_to_ed25519,
-                    connect_ecdsa_to_ed448,
-                    connect_ed25519_to_dsa,
+-define(tests_new, [connect_ed25519_to_dsa,
                     connect_ed25519_to_ecdsa,
                     connect_ed25519_to_ed448,
                     connect_ed25519_to_ed25519,
@@ -132,9 +137,7 @@ all() ->
                     connect_ed448_to_ecdsa,
                     connect_ed448_to_ed25519,
                     connect_ed448_to_ed448,
-                    connect_ed448_to_rsa_sha2,
-                    connect_rsa_sha2_to_ed25519,
-                    connect_rsa_sha2_to_ed448
+                    connect_ed448_to_rsa_sha2
                     | ?tests_old % but taken from the new format directory
                    ]).
 
@@ -158,7 +161,8 @@ groups() ->
        ssh_rfc4716_rsa_comment, ssh_rfc4716_dsa_comment,
        ssh_rfc4716_rsa_subject,
        ssh_list_public_key,
-%%       ssh_known_hosts, ssh_auth_keys, ssh1_known_hosts, ssh1_auth_keys,
+       ssh_known_hosts, %% ssh1_known_hosts,
+       ssh_auth_keys, %% ssh1_auth_keys,
        ssh_openssh_key_with_comment,
        ssh_openssh_key_long_header]}
     ].
@@ -564,10 +568,10 @@ ssh_list_public_key(Config) when is_list(Config) ->
 chk_encode(Data, Type) ->
     case ssh_file:decode(ssh_file:encode(Data,Type), Type) of
         Data->
-            ct:pal("re-encode ~p ok", [Type]),
+            ct:log("re-encode ~p ok", [Type]),
             true;
         Result ->
-            ct:pal("re-encode ~p FAILED~n"
+            ct:log("re-encode ~p FAILED~n"
                    "Got~n ~p~nExpect~n ~p~n",
                    [Type, Result, Data]),
             false
@@ -577,10 +581,10 @@ chk_encode(Data, Type) ->
 chk_decode(Data, Expect, Type) ->
     case ssh_file:decode(Data, Type) of
         Expect ->
-            ct:pal("decode ~p ok", [Type]),
+            ct:log("decode ~p ok", [Type]),
             true;
         BadResult ->
-            ct:pal("decode ~p FAILED~n"
+            ct:log("decode ~p FAILED~n"
                    "Result~n ~p~nExpect~n ~p~n"
                    "~p",
                    [Type, BadResult, Expect,
@@ -783,6 +787,54 @@ ssh_hostkey(rsa) ->
 	  public_key),
     PKdecoded.
 
+%%%----------------------------------------------------------------
+chk_known_hosts(Config) ->
+    PrivDir = proplists:get_value(priv_dir, Config),
+
+    DataDir = filename:join(proplists:get_value(data_dir,Config), "new_format"),
+    SysDir = filename:join(PrivDir, "chk_known_hosts_sys_dir"),
+    ssh_test_lib:setup_all_host_keys(DataDir, SysDir),
+
+    UsrDir = filename:join(PrivDir, "chk_known_hosts_usr_dir"),
+    file:make_dir(UsrDir),
+    KnownHostsFile = filename:join(UsrDir, "known_hosts"),
+
+    DaemonOpts = [{system_dir, SysDir},
+                  {user_dir, UsrDir},
+                  {password, "bar"}],
+
+    UserOpts = [{user_dir, UsrDir},
+                {user, "foo"},
+                {password, "bar"},
+                {silently_accept_hosts, true},
+                {user_interaction, false}
+               ],
+
+    {_Pid1, Host1, Port1} = ssh_test_lib:daemon(DaemonOpts),
+    {_Pid2, Host2, Port2} = ssh_test_lib:daemon(DaemonOpts),
+
+    _C1 = ssh_test_lib:connect(Host1, Port1, UserOpts),
+    {ok,KnownHosts1} = file:read_file(KnownHostsFile),
+    Sz1 = byte_size(KnownHosts1),
+    ct:log("~p bytes KnownHosts1 = ~p", [Sz1, KnownHosts1]),
+
+    _C2 = ssh_test_lib:connect(Host2, Port2, UserOpts),
+    {ok,KnownHosts2} = file:read_file(KnownHostsFile),
+    Sz2 = byte_size(KnownHosts2),
+    ct:log("~p bytes KnownHosts2 = ~p", [Sz2, KnownHosts2]),
+
+    %% Check that 2nd is appended after the 1st:
+    <<KnownHosts1:Sz1/binary, _/binary>> = KnownHosts2,
+
+    %% Check that there are exactly two NLs:
+    2 = lists:foldl(fun($\n, Sum) -> Sum + 1;
+                       (_,   Sum) -> Sum
+                    end, 0, binary_to_list(KnownHosts2)),
+
+    %% Check that at least one NL terminates both two lines:
+    <<_:(Sz1-1)/binary, $\n, _:(Sz2-Sz1-1)/binary, $\n>> = KnownHosts2.
+
+
 %%%----------------------------------------------------------------
 try_connect({skip,Reson}) ->
     {skip,Reson};
